@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: butterfly.c,v 1.12 2005/10/20 11:36:19 joerg_wunsch Exp $ */
+/* $Id: butterfly.c,v 1.14 2006/09/01 10:03:12 joerg_wunsch Exp $ */
 
 /*
  * avrdude interface for the serial programming mode of the Atmel butterfly
@@ -227,7 +227,6 @@ static int butterfly_initialize(PROGRAMMER * pgm, AVRPART * p)
   char buf[10];
   char type;
   char c;
-  int dev_supported = 0;
 
   no_show_func_info();
 
@@ -313,22 +312,9 @@ static int butterfly_initialize(PROGRAMMER * pgm, AVRPART * p)
     if (c == 0)
       break;
     fprintf(stderr, "    Device code: 0x%02x\n", (unsigned int)(unsigned char)c);
-
-    /* FIXME: Need to lookup devcode and report the device. */
-
-    if (p->avr910_devcode == (int)(unsigned char)c)
-      dev_supported = 1;
   };
   fprintf(stderr,"\n");
 
-  if (!dev_supported) {
-    /* FIXME: if nothing matched, we should rather compare the device
-       signatures. */
-    fprintf(stderr,
-            "%s: error: selected device is not supported by programmer: %s\n",
-            progname, p->id);
-  }
-
   /* Tell the programmer which part we selected. */
 
   buf[0] = 'T';
@@ -337,10 +323,9 @@ static int butterfly_initialize(PROGRAMMER * pgm, AVRPART * p)
   butterfly_send(pgm, buf, 2);
   butterfly_vfy_cmd_sent(pgm, "select device");
 
-  if (dev_supported)
-      butterfly_enter_prog_mode(pgm);
+  butterfly_enter_prog_mode(pgm);
 
-  return dev_supported? 0: -1;
+  return 0;
 }
 
 
@@ -389,8 +374,9 @@ static void butterfly_close(PROGRAMMER * pgm)
 {
   no_show_func_info();
 
-  /* "exit programmer" added by Martin Thomas 2/2004 */
+  /* "exit programmer" */
   butterfly_send(pgm, "E", 1);
+  butterfly_vfy_cmd_sent(pgm, "exit bootloader");
 
   serial_close(pgm->fd);
   pgm->fd = -1;
