@@ -17,18 +17,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: par.h,v 1.4 2005/09/18 20:12:23 joerg_wunsch Exp $ */
+/* $Id: par.h,v 1.5 2005/11/01 23:02:06 joerg_wunsch Exp $ */
 
 #ifndef __par_h__
 #define __par_h__
 
 void par_initpgm        (PROGRAMMER * pgm);
 
-int par_getpinmask(int pin);
-int par_setpin(int fd, int pin, int value);
-int par_getpin(int fd, int pin);
-int par_highpulsepin(int fd, int pin);
-
 #endif
 
 
