@@ -2,6 +2,7 @@
  * avrdude - A Downloader/Uploader for AVR device programmers
  * Copyright (C) 2000, 2001, 2002, 2003  Brian S. Dean <bsd@bsdhome.com>
  * Copyright (C) 2005 Michael Holzt <kju-avr@fqdn.org>
+ * Copyright (C) 2006 Joerg Wunsch <j@uriah.heep.sax.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: serbb_posix.c,v 1.2 2005/11/01 23:02:06 joerg_wunsch Exp $ */
+/* $Id: serbb_posix.c,v 1.8 2006/09/01 10:06:53 joerg_wunsch Exp $ */
 
 /*
  * Posix serial bitbanging interface for avrdude.
@@ -50,25 +51,30 @@ struct termios oldmode;
   serial port/pin mapping
 
   1	cd	<-
-  2	rxd	<-
+  2	(rxd)	<-
   3	txd	->
   4	dtr	->
-  5	dsr	<-
-  6	rts	->
-  7	cts	<-
+  5	GND
+  6	dsr	<-
+  7	rts	->
+  8	cts	<-
+  9	ri	<-
 */
 
-static int serregbits[] =
-{ TIOCM_CD, 0, 0, TIOCM_DTR, TIOCM_DSR, TIOCM_RTS, TIOCM_CTS };
+#define DB9PINS 9
+
+static int serregbits[DB9PINS + 1] =
+{ 0, TIOCM_CD, 0, 0, TIOCM_DTR, 0, TIOCM_DSR, TIOCM_RTS, TIOCM_CTS, TIOCM_RI };
 
 #ifdef DEBUG
-static char *serpins[7] =
-  { "CD", "RXD", "TXD ~RESET", "DTR MOSI", "DSR", "RTS SCK", "CTS MISO" };
+static char *serpins[DB9PINS + 1] =
+  { "NONE", "CD", "RXD", "TXD", "DTR", "GND", "DSR", "RTS", "CTS", "RI" };
 #endif
 
 static int serbb_setpin(PROGRAMMER * pgm, int pin, int value)
 {
   unsigned int	ctl;
+  int           r;
 
   if (pin & PIN_INVERSE)
   {
@@ -76,28 +82,39 @@ static int serbb_setpin(PROGRAMMER * pgm, int pin, int value)
     pin   &= PIN_MASK;
   }
 
-  if ( pin < 1 || pin > 7 )
+  if ( pin < 1 || pin > DB9PINS )
     return -1;
 
-  pin--;
-
 #ifdef DEBUG
   printf("%s to %d\n",serpins[pin],value);
 #endif
 
   switch ( pin )
   {
-    case 2:  /* txd */
-             ioctl(pgm->fd, value ? TIOCSBRK : TIOCCBRK, 0);
+    case 3:  /* txd */
+	     r = ioctl(pgm->fd, value ? TIOCSBRK : TIOCCBRK, 0);
+	     if (r < 0) {
+	       perror("ioctl(\"TIOCxBRK\")");
+	       return -1;
+	     }
              return 0;
 
-    case 3:  /* dtr, rts */
-    case 5:  ioctl(pgm->fd, TIOCMGET, &ctl);
+    case 4:  /* dtr */
+    case 7:  /* rts */
+             r = ioctl(pgm->fd, TIOCMGET, &ctl);
+ 	     if (r < 0) {
+	       perror("ioctl(\"TIOCMGET\")");
+	       return -1;
+ 	     }
              if ( value )
                ctl |= serregbits[pin];
              else
                ctl &= ~(serregbits[pin]);
-             ioctl(pgm->fd, TIOCMSET, &ctl);
+	     r = ioctl(pgm->fd, TIOCMSET, &ctl);
+ 	     if (r < 0) {
+	       perror("ioctl(\"TIOCMSET\")");
+	       return -1;
+ 	     }
              return 0;
 
     default: /* impossible */
@@ -109,6 +126,7 @@ static int serbb_getpin(PROGRAMMER * pgm, int pin)
 {
   unsigned int	ctl;
   unsigned char invert;
+  int           r;
 
   if (pin & PIN_INVERSE)
   {
@@ -117,21 +135,23 @@ static int serbb_getpin(PROGRAMMER * pgm, int pin)
   } else
     invert = 0;
 
-  if ( pin < 1 || pin > 7 )
+  if ( pin < 1 || pin > DB9PINS )
     return(-1);
 
-  pin --;
-
   switch ( pin )
   {
-    case 1:  /* rxd, currently not implemented, FIXME */
+    case 2:  /* rxd, currently not implemented, FIXME */
              return(-1);
 
-    case 0:  /* cd, dsr, dtr, rts, cts */
-    case 3:
-    case 4:
-    case 5:
-    case 6:  ioctl(pgm->fd, TIOCMGET, &ctl);
+    case 1:  /* cd  */
+    case 6:  /* dsr */
+    case 8:  /* cts */
+    case 9:  /* ri  */
+             r = ioctl(pgm->fd, TIOCMGET, &ctl);
+ 	     if (r < 0) {
+	       perror("ioctl(\"TIOCMGET\")");
+	       return -1;
+ 	     }
              if ( !invert )
              {
 #ifdef DEBUG
@@ -154,18 +174,16 @@ static int serbb_getpin(PROGRAMMER * pgm, int pin)
 
 static int serbb_highpulsepin(PROGRAMMER * pgm, int pin)
 {
-  if (pin < 1 || pin > 7)
+  if ( pin < 1 || pin > DB9PINS )
     return -1;
 
   serbb_setpin(pgm, pin, 1);
-  #if SLOW_TOGGLE
-  usleep(1000);
-  #endif
-  serbb_setpin(pgm, pin, 0);
+  if (pgm->ispdelay > 1)
+    bitbang_delay(pgm->ispdelay);
 
-  #if SLOW_TOGGLE
-  usleep(1000);
-  #endif
+  serbb_setpin(pgm, pin, 0);
+  if (pgm->ispdelay > 1)
+    bitbang_delay(pgm->ispdelay);
 
   return 0;
 }
@@ -201,15 +219,25 @@ static int serbb_open(PROGRAMMER *pgm, char *port)
 {
   struct termios mode;
   int flags;
+  int r;
+
+  bitbang_check_prerequisites(pgm);
 
   /* adapted from uisp code */
 
   pgm->fd = open(port, O_RDWR | O_NOCTTY | O_NONBLOCK);
 
-  if (pgm->fd < 0)
+  if (pgm->fd < 0) {
+    perror(port);
     return(-1);
+  }
 
-  tcgetattr(pgm->fd, &mode);
+  r = tcgetattr(pgm->fd, &mode);
+  if (r < 0) {
+    fprintf(stderr, "%s: ", port);
+    perror("tcgetattr");
+    return(-1);
+  }
   oldmode = mode;
 
   mode.c_iflag = IGNBRK | IGNPAR;
@@ -218,19 +246,26 @@ static int serbb_open(PROGRAMMER *pgm, char *port)
   mode.c_cc [VMIN] = 1;
   mode.c_cc [VTIME] = 0;
 
-  tcsetattr(pgm->fd, TCSANOW, &mode);
+  r = tcsetattr(pgm->fd, TCSANOW, &mode);
+  if (r < 0) {
+      fprintf(stderr, "%s: ", port);
+      perror("tcsetattr");
+      return(-1);
+  }
 
   /* Clear O_NONBLOCK flag.  */
   flags = fcntl(pgm->fd, F_GETFL, 0);
   if (flags == -1)
     {
-      fprintf(stderr, "%s: Can not get flags\n", progname);
+      fprintf(stderr, "%s: Can not get flags: %s\n",
+	      progname, strerror(errno));
       return(-1);
     }
   flags &= ~O_NONBLOCK;
   if (fcntl(pgm->fd, F_SETFL, flags) == -1)
     {
-      fprintf(stderr, "%s: Can not clear nonblock flag\n", progname);
+      fprintf(stderr, "%s: Can not clear nonblock flag: %s\n",
+	      progname, strerror(errno));
       return(-1);
     }
 
@@ -239,7 +274,12 @@ static int serbb_open(PROGRAMMER *pgm, char *port)
 
 static void serbb_close(PROGRAMMER *pgm)
 {
-  tcsetattr(pgm->fd, TCSANOW, &oldmode);
+  if (pgm->fd != -1)
+  {
+	  (void)tcsetattr(pgm->fd, TCSANOW, &oldmode);
+	  pgm->setpin(pgm, pgm->pinno[PIN_AVR_RESET], 1);
+	  close(pgm->fd);
+  }
   return;
 }
 
