@@ -2,7 +2,7 @@
  * avrdude - A Downloader/Uploader for AVR device programmers
  * Copyright (C) 2003, 2004  Martin J. Thomas  <mthomas@rhrk.uni-kl.de>
  * Copyright (C) 2005 Michael Holzt <kju-avr@fqdn.org>
- * Copyright (C) 2005 Joerg Wunsch <j@uriah.heep.sax.de>
+ * Copyright (C) 2005, 2006 Joerg Wunsch <j@uriah.heep.sax.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* $Id: serbb_win32.c,v 1.2 2005/11/01 23:02:06 joerg_wunsch Exp $ */
+/* $Id: serbb_win32.c,v 1.6 2006/08/31 10:55:20 joerg_wunsch Exp $ */
 
 /*
  * Win32 serial bitbanging interface for avrdude.
@@ -49,16 +49,18 @@ static int dtr, rts, txd;
   serial port/pin mapping
 
   1	cd	<-
-  2	rxd	<-
+  2	(rxd)	<-
   3	txd	->
   4	dtr	->
-  5	dsr	<-
-  6	rts	->
-  7	cts	<-
-
-  Negative pin # means negated value.
+  5	GND
+  6	dsr	<-
+  7	rts	->
+  8	cts	<-
+  9	ri	<-
 */
 
+#define DB9PINS 9
+
 static int serbb_setpin(PROGRAMMER * pgm, int pin, int value)
 {
 	HANDLE hComPort = (HANDLE)pgm->fd;
@@ -72,26 +74,24 @@ static int serbb_setpin(PROGRAMMER * pgm, int pin, int value)
                 pin &= PIN_MASK;
         }
 
-        if (pin < 1 || pin > 7)
+        if (pin < 1 || pin > DB9PINS)
                 return -1;
 
-        pin--;
-
         switch (pin)
         {
-        case 2:  /* txd */
+        case 3:  /* txd */
                 dwFunc = value? SETBREAK: CLRBREAK;
                 name = value? "SETBREAK": "CLRBREAK";
                 txd = value;
                 break;
 
-        case 3:  /* dtr */
+        case 4:  /* dtr */
                 dwFunc = value? SETDTR: CLRDTR;
                 name = value? "SETDTR": "CLRDTR";
                 dtr = value;
                 break;
 
-        case 5:  /* rts */
+        case 7:  /* rts */
                 dwFunc = value? SETRTS: CLRRTS;
                 name = value? "SETRTS": "CLRRTS";
                 break;
@@ -144,12 +144,10 @@ static int serbb_getpin(PROGRAMMER * pgm, int pin)
         } else
                 invert = 0;
 
-        if (pin < 1 || pin > 7)
+        if (pin < 1 || pin > DB9PINS)
                 return -1;
 
-        pin --;
-
-        if (pin == 0 /* cd */ || pin == 4 /* dsr */ || pin == 6 /* cts */)
+        if (pin == 1 /* cd */ || pin == 6 /* dsr */ || pin == 8 /* cts */)
         {
                 if (!GetCommModemStatus(hComPort, &modemstate))
                 {
@@ -176,13 +174,13 @@ static int serbb_getpin(PROGRAMMER * pgm, int pin)
                                 progname, modemstate);
                 switch (pin)
                 {
-                case 0:
+                case 1:
                         modemstate &= MS_RLSD_ON;
                         break;
-                case 4:
+                case 6:
                         modemstate &= MS_DSR_ON;
                         break;
-                case 6:
+                case 8:
                         modemstate &= MS_CTS_ON;
                         break;
                 }
@@ -195,15 +193,15 @@ static int serbb_getpin(PROGRAMMER * pgm, int pin)
 
         switch (pin)
         {
-        case 2: /* txd */
+        case 3: /* txd */
                 rv = txd;
                 name = "TXD";
                 break;
-        case 3: /* dtr */
+        case 4: /* dtr */
                 rv = dtr;
                 name = "DTR";
                 break;
-        case 5: /* rts */
+        case 7: /* rts */
                 rv = rts;
                 name = "RTS";
                 break;
@@ -230,14 +228,12 @@ static int serbb_highpulsepin(PROGRAMMER * pgm, int pin)
                 return -1;
 
         serbb_setpin(pgm, pin, 1);
-#if SLOW_TOGGLE
-        usleep(1000);
-#endif
-        serbb_setpin(pgm, pin, 0);
+	if (pgm->ispdelay > 1)
+	  bitbang_delay(pgm->ispdelay);
 
-#if SLOW_TOGGLE
-        usleep(1000);
-#endif
+        serbb_setpin(pgm, pin, 0);
+	if (pgm->ispdelay > 1)
+	  bitbang_delay(pgm->ispdelay);
 
         return 0;
 }
@@ -274,6 +270,8 @@ static int serbb_open(PROGRAMMER *pgm, char *port)
 	LPVOID lpMsgBuf;
 	HANDLE hComPort = INVALID_HANDLE_VALUE;
 
+	bitbang_check_prerequisites(pgm);
+
 	hComPort = CreateFile(port, GENERIC_READ | GENERIC_WRITE, 0, NULL,
                               OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 
@@ -336,7 +334,10 @@ static void serbb_close(PROGRAMMER *pgm)
 {
 	HANDLE hComPort=(HANDLE)pgm->fd;
 	if (hComPort != INVALID_HANDLE_VALUE)
+	{
+		pgm->setpin(pgm, pgm->pinno[PIN_AVR_RESET], 1);
 		CloseHandle (hComPort);
+	}
         if (verbose > 2)
                 fprintf(stderr,
                         "%s: ser_close(): closed comm port handle 0x%x\n",
